/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
  *
  * Condor Software Copyright Notice
  * Copyright (C) 1990-2005, Condor Team, Computer Sciences Department,
  * University of Wisconsin-Madison, WI.
  *
  * This source code is covered by the Condor Public License, which can
  * be found in the accompanying LICENSE.TXT file, or online at
  * www.condorproject.org.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  * AND THE UNIVERSITY OF WISCONSIN-MADISON "AS IS" AND ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
  * WARRANTIES OF MERCHANTABILITY, OF SATISFACTORY QUALITY, AND FITNESS
  * FOR A PARTICULAR PURPOSE OR USE ARE DISCLAIMED. THE COPYRIGHT
  * HOLDERS AND CONTRIBUTORS AND THE UNIVERSITY OF WISCONSIN-MADISON
  * MAKE NO MAKE NO REPRESENTATION THAT THE SOFTWARE, MODIFICATIONS,
  * ENHANCEMENTS OR DERIVATIVE WORKS THEREOF, WILL NOT INFRINGE ANY
  * PATENT, COPYRIGHT, TRADEMARK, TRADE SECRET OR OTHER PROPRIETARY
  * RIGHT.
  *
  ****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/

#ifndef NEST_CONCURRENCY_MANAGER_H
#define NEST_CONCURRENCY_MANAGER_H

#include "nest_common.h"
#include "nest_client.h"
#include "nest_concurrency_model.h"

// Because of interdependency hell, we need to forward define these...
class nestStorageManager;
class nestProtocolManager;
class nestConcurrencyModel;
class
nestConcurrencyManager {

  public:

	nestConcurrencyManager::nestConcurrencyManager (
        fd_set *readfds,
		fd_set *writefds,
		fd_set *r_ready,
		fd_set *w_ready,
        void( *notifyServerCallback )(nestClient *, NestReplyStatus ),
		nestStorageManager *storage_manager,
		nestProtocolManager *protocol_manager
		);
	nestConcurrencyManager::~nestConcurrencyManager ( void );

    int init( void );

    void sigChild( int signal, siginfo_t *info, void *context );

    void makeProgress( );

    void taskDone(nestClient *client, NestReplyStatus status, int model);
    
    NestReplyStatus addTask( nestClient *client );
    
    int getMaxfd( ); 

    char *getConcurrencyModel( );

    void debugLevel( long level );
    
    int processFunction( int id );

    ssize_t writeAd( nestProtocol *ad );

  private:
	nestStorageManager *storage_manager;
	nestProtocolManager *protocol_manager;
    void setUpModelList( );

    nestConcurrencyModel *selectConcurrencyModel( nestClient *client );
    nestConcurrencyModel *selectUsingAverages( long transferlength );
    nestConcurrencyModel *selectTakingTurns( );

    nestClient *selectFavoriteProtocol( );
    nestClient *selectShortestFile( );
    nestClient *selectCachedFile();

    nestClient *selectLowestStride( );
    unsigned long findLowestStride( );
    int makeSchedule();
    NestReplyStatus makeStrides();
    void strideDone( nestClient *, long len );

    NestReplyStatus lockMutex();
    NestReplyStatus unlockMutex();

    
    nestClient *selectQueuedTask( );

    NestReplyStatus startTask( nestClient *client );
    
    void filecache_model_check( char *file, 
								nestFiletransfer *transfer,
								int model_id);

    char * modelName( int model );
    
};


#endif
