/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
  *
  * Condor Software Copyright Notice
  * Copyright (C) 1990-2005, Condor Team, Computer Sciences Department,
  * University of Wisconsin-Madison, WI.
  *
  * This source code is covered by the Condor Public License, which can
  * be found in the accompanying LICENSE.TXT file, or online at
  * www.condorproject.org.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  * AND THE UNIVERSITY OF WISCONSIN-MADISON "AS IS" AND ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
  * WARRANTIES OF MERCHANTABILITY, OF SATISFACTORY QUALITY, AND FITNESS
  * FOR A PARTICULAR PURPOSE OR USE ARE DISCLAIMED. THE COPYRIGHT
  * HOLDERS AND CONTRIBUTORS AND THE UNIVERSITY OF WISCONSIN-MADISON
  * MAKE NO MAKE NO REPRESENTATION THAT THE SOFTWARE, MODIFICATIONS,
  * ENHANCEMENTS OR DERIVATIVE WORKS THEREOF, WILL NOT INFRINGE ANY
  * PATENT, COPYRIGHT, TRADEMARK, TRADE SECRET OR OTHER PROPRIETARY
  * RIGHT.
  *
  ****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/

#ifndef NEST_FILECACHE_MODEL_H
#define NEST_FILECACHE_MODEL_H

#include <stdio.h>
#include <assert.h>
#include <sys/time.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>

#include "nest_common.h"
#include "nest_filecache_entry.h"
#include "nest_hash_table.h"
#include "nest_protocol.h"

#define INFINITE_IMPORTANCE -1

typedef NestReplyStatus (*removeCallback) ( const char *file );

class nestFilecacheModel {

public:

    nestFilecacheModel( );
    
    nestFilecacheModel( float megs, int type, 
            removeCallback cb = NULL, bool track_hits = false );

    virtual NestReplyStatus insert( bool *hit, const char *file, 
                                    long size, int weight = 1 );

    virtual bool remove( const char *file );
    
    void refresh( const char *file );

    virtual bool check( const char *file );

    nestFilecacheEntry * check_and_refresh( const char *file );

    void debug( );

    virtual bool grow( float megs );

    virtual bool shrink( float megs );

    virtual bool renameFile( const char *oldname, const char *newname );

    virtual float hit_rate( );
    
    virtual long gethits( );

    virtual long getmisses( );
    
    virtual long getevictions( );
    
    virtual float getsize( );

    virtual long getavail( );

    virtual long getFilesize( const char *file );

    virtual void setFilesize( const char *file, long size );

    virtual void writeAd( nestProtocol *ad );

    virtual NestReplyStatus format();

    virtual bool rememberLength( const char *file, long *length );
    
private:

    nestFilecacheEntry *head, *tail;
    removeCallback removeCb;
    struct nest_hash_table *ht;

    long size, avail;   // in KB
    long entries;
    bool use_hashtable;
    bool log_misses;
    int  type;

    struct nest_hash_table *infinite_cache;
    nestProtocol *cacheLog;
    unsigned long hits, compulsory_misses, evictions, capacity_misses;

    void logMiss( const char *file );
    long make_space( long size );
    long freeEntry( nestFilecacheEntry * entry );
    bool validityCheck();

    nestFilecacheEntry * pushEntry( nestFilecacheEntry * entry );
    nestFilecacheEntry * removeEntry( nestFilecacheEntry * entry );
    nestFilecacheEntry * findEntry( const char *file );
};

#endif
