/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
  *
  * Condor Software Copyright Notice
  * Copyright (C) 1990-2005, Condor Team, Computer Sciences Department,
  * University of Wisconsin-Madison, WI.
  *
  * This source code is covered by the Condor Public License, which can
  * be found in the accompanying LICENSE.TXT file, or online at
  * www.condorproject.org.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  * AND THE UNIVERSITY OF WISCONSIN-MADISON "AS IS" AND ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
  * WARRANTIES OF MERCHANTABILITY, OF SATISFACTORY QUALITY, AND FITNESS
  * FOR A PARTICULAR PURPOSE OR USE ARE DISCLAIMED. THE COPYRIGHT
  * HOLDERS AND CONTRIBUTORS AND THE UNIVERSITY OF WISCONSIN-MADISON
  * MAKE NO MAKE NO REPRESENTATION THAT THE SOFTWARE, MODIFICATIONS,
  * ENHANCEMENTS OR DERIVATIVE WORKS THEREOF, WILL NOT INFRINGE ANY
  * PATENT, COPYRIGHT, TRADEMARK, TRADE SECRET OR OTHER PROPRIETARY
  * RIGHT.
  *
  ****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/

#include "nest_filecache_model_mgr.h"


nestFilecacheModelManager::nestFilecacheModelManager( int type ) {
    this->cachetype = type;

    cachehit_guesses = cachemiss_guesses = 0;
    false_hits = false_misses = 0;
    correct_hits = correct_misses = 0;

    cache_initial_size  = DEFAULT_CACHE_SIZE;
    cache_threshold     = DEFAULT_CACHE_THRESHOLD;
    growth_size         = DEFAULT_GROWTH_SIZE;

    cacheguesslog = cachesizelog = NULL;

        // open our logfiles
    /*
    cacheguesslog   = nestStorageManager::openMetaFile("cache.guesses", 
            WRITE_FLAGS | O_TRUNC );
    cachesizelog   = nestStorageManager::openMetaFile("cache.sizes", 
            WRITE_FLAGS | O_TRUNC );
    */
    cachesizelog  = NULL;
    cacheguesslog = NULL;

        // create our model
    model = new nestFilecacheModel( cache_initial_size, cachetype, NULL, false);

}

int 
nestFilecacheModelManager::taskDone( 
        const char *file, 
        nestFiletransfer *transfer, 
        int concurrency_model,
        int current_tasks )
{

    bool do_nothing = false;
    if ( do_nothing ) return 0;

        // with small files, we'll see a whole lot of small hits . .. 

    bool guess =  transfer->getCacheGuess();
    float bw   = transfer->getBandwidth();
    long  len  = transfer->getBytesTransferred();
    struct IO_Measurement *measurements = transfer->getMeasurements();
    float first_bw = NestBandwidth( (long)measurements->first_in_size,
                                (long)measurements->first_in_time );
    float tot_bw   = NestBandwidth( (long)len, (long)
            measurements->total_in_time );
    char log[NEST_MESSAGESIZE];
    struct timeval now;
    bool sizechange = false;
    float cachesize = 0.0;
    char *miss = "miss";
    char *hit  = "hit ";
    char *guess_string;
    char equality_sign;

    NestGettimeofday( &now );

        // log the cache measurements
    if ( cacheguesslog != NULL ) {
        NestSnprintf( log, 1024, "%2d %2d %7.3f %7.3f %7.3f %3d %s\n", 
                guess, 
                concurrency_model,
                bw,
                first_bw,
                tot_bw,
                current_tasks,
                file );
        cacheguesslog->Sendn( log, NestStrlen( log ) );
    }

    float use_bw = tot_bw;

        // adjust cachesize if necessary if the guess was wrong or
        // if the initially given filesize was incorrect
        // (this may happen because the filesize was initially unknown)
    if ( len > model->getFilesize( file ) ) {
        model->shrink( 
                (float)( len - model->getFilesize( file ) ) / (1024*1024) );
        model->setFilesize( file, len );
    }
    
    if ( guess ) {
        use_bw = first_bw;
        guess_string = hit;
        equality_sign = '<';
        if ( use_bw < cache_threshold ) {
            sizechange = true;
            model->shrink( growth_size );
            false_hits++;
        } else {
            correct_hits++;
        }
    } else if ( ! guess ) {
        use_bw = first_bw;
        guess_string = miss;
        equality_sign = '>';
        if ( use_bw > cache_threshold ) {
            sizechange = true;
            model->grow( growth_size );
            false_misses++;
        } else {
            correct_misses++;
        }
    } else {
        assert( 0 );
    }

    if ( sizechange ) {
        cachesize = (float) ( (float)model->getsize() / ( 1024 * 1024 ) );
        if ( cachesizelog != NULL ) {
            NestSnprintf( log, 1024, "%ld.%ld %.3f\n",
                now.tv_sec, now.tv_usec, cachesize );
            cachesizelog->Sendn( log, NestStrlen( log ) );
        }
        nest_debug( NEST_DEBUG_CACHEMODEL, 
                "False %5s %10s"
                "%6.2f [%6.2f/%6.2f/%6.2f] %c %4.1f): cache -> %6.1f\n",
                guess_string,
                file,
                use_bw,
                bw,
                first_bw,
                tot_bw,
                equality_sign,
                cache_threshold,
                cachesize );
    }

    return 0;
}

bool 
nestFilecacheModelManager::contains( const char *file ) {
    return model->check_and_refresh( file );
}

bool 
nestFilecacheModelManager::insert(  const char *file,  
                                    nestFiletransfer *transfer, 
                                    long filesize ) 
{
    if ( filesize < 0 ) {
            // this means filesize is unknown
            // we probably still want to know that the file is in cache
            // we will have to delay victimizing until we get 
            // the taskDone operation
        filesize = 0;
    }

    /* the transfer object might be NULL in case of a 3pt */
    
    bool hit = false;
    NestReplyStatus status = model->insert( &hit, file, filesize );
    if ( status != NEST_SUCCESS ) {
        nest_warn( __FILE__, __FUNCTION__, "Failed to insert %s (%ld bytes) into cache: %s\n",
                file, filesize, NestErrorString( status ) );
    } else {
        ( hit ) ? cachehit_guesses++ : cachemiss_guesses++;
    }

    if ( transfer ) transfer->setCacheGuess( hit );
    return hit;
}

ssize_t
nestFilecacheModelManager::writeAd( nestProtocol *ad ) {
    char buffer[NEST_MESSAGESIZE];
    int ret = NestSnprintf( buffer, NEST_MESSAGESIZE, 
        "CacheSize        = %9.0f\n"
        "CacheHits        = %9ld\n"
        "CacheMisses      = %9ld\n"
        "CachePercentHits = %.3f\n"
        "FalseHits        = %9ld\n"
        "FalseMisses      = %9ld\n"
        "CorrectHits      = %9ld\n"
        "CorrectMisses    = %9ld\n",
        (float)( model->getsize() / (1024*1024)),
        cachehit_guesses, 
        cachemiss_guesses,
        ((float)cachehit_guesses/(cachehit_guesses+cachemiss_guesses)*100),
        false_hits, 
        false_misses, 
        correct_hits, 
        correct_misses );

    if ( ret < 0 ) {
        nest_warn( __FILE__, __FUNCTION__, "buffer truncated in %s:%s.\n", __FILE__, __FUNCTION__ );
    }

    return ad->Sendn( buffer, NestStrlen( buffer ) );
}
