/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
  *
  * Condor Software Copyright Notice
  * Copyright (C) 1990-2005, Condor Team, Computer Sciences Department,
  * University of Wisconsin-Madison, WI.
  *
  * This source code is covered by the Condor Public License, which can
  * be found in the accompanying LICENSE.TXT file, or online at
  * www.condorproject.org.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  * AND THE UNIVERSITY OF WISCONSIN-MADISON "AS IS" AND ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
  * WARRANTIES OF MERCHANTABILITY, OF SATISFACTORY QUALITY, AND FITNESS
  * FOR A PARTICULAR PURPOSE OR USE ARE DISCLAIMED. THE COPYRIGHT
  * HOLDERS AND CONTRIBUTORS AND THE UNIVERSITY OF WISCONSIN-MADISON
  * MAKE NO MAKE NO REPRESENTATION THAT THE SOFTWARE, MODIFICATIONS,
  * ENHANCEMENTS OR DERIVATIVE WORKS THEREOF, WILL NOT INFRINGE ANY
  * PATENT, COPYRIGHT, TRADEMARK, TRADE SECRET OR OTHER PROPRIETARY
  * RIGHT.
  *
  ****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/

#ifndef _NEST_PROTOCOL_GRID_FTP_H
#define _NEST_PROTOCOL_GRID_FTP_H

#include "nest_common.h"
#include "nest_protocol.h"
#include "nest_globus.h"

#include "nest_storage_manager.h"

#ifndef HAVE_PTHREAD
// if we're using the non-threaded globus libs
#include "nest_pthread_util.h"
#endif


class nestProtocolGridFtp : public nestProtocol {
public:
    static int listen( int port );

    static nestProtocol* accept( int socket );
    
    nestProtocolGridFtp( globus_ftp_control_handle_t *handle, int id ); 
    
    virtual ~nestProtocolGridFtp();
    
    virtual bool sendPwd( const char *pwd );
    
    virtual bool sendDirectoryList(  NestFileInfo *list );
    
    virtual bool sendErrorMessage(  const char* filename,
									const char* func_name, int error );
    
    virtual bool sendFinalAck(  int transferType );

    virtual bool sendTransferInfo(  nestRequest *req );

    virtual bool sendFilesize( nestFileSize size);

    virtual nestRequest* receiveRequest(  fd_set *ready );

    virtual ssize_t Receive( char *buffer, ssize_t n );

    virtual ssize_t Send( const char *buffer, ssize_t n );
    
    virtual int getType( ) { return GFTP_PROTOCOL; }

    virtual bool hasSocket() { return false; }

    virtual NestReplyStatus gssAccept( char **username, char **globusname );

    int getSocket() { return -1; }
    
    virtual bool endThirdPartyTransfer( int ) { assert( 0 ); return false; }

    globus_ftp_control_handle_t * getHandle();

	const char *getName( void ) { return "GridFTP"; };

private:

    globus_ftp_control_handle_t     *handle;
    nestRequest                     *request;
    char                            *username;
    char                            *globusname;
    int                             id;
    int                             reqType;
    char                            buffer[FILEBUFFER];
    int                             bufsize;
    int                             transmissionLength;
    bool                            isNew;
    bool                            eof;
    globus_off_t                    offset;
    gss_cred_id_t                   delegated_credential_handle;
    globus_result_t                 transferStatus;
    globus_bool_t                   authResult;
    globus_bool_t                   acceptResult;

    bool isNewConnection();
    
    static globus_fifo_t fifo;
    static pthread_mutex_t fifo_mutex;
    static int pipes[2];
 
    static void commandCallback( void *arg, 
            struct globus_ftp_control_handle_s *handle,
            globus_object_t *error );

    static void closeCallback( void *arg, 
            struct globus_ftp_control_handle_s *handle,
            globus_object_t *error,
            globus_ftp_control_response_t *response );
    
    static void controlDataCallback( void *callback_arg, 
                    globus_ftp_control_handle_t * handle, 
                    globus_object_t * error, 
                    globus_byte_t * buffer, 
                    globus_size_t length, 
                    globus_off_t offset, 
                    globus_bool_t eof );
    
    static void signalNest();

    static void clearNestSignal();
    
    static void * GlobusCondWait( void *);

    static char * globusResultToString( globus_result_t result );

    static void dataConnectCallback(  void *callback_arg,
                        struct globus_ftp_control_handle_s *handle,
                        unsigned int stripe_ndx,
                        globus_bool_t reused,
                        globus_object_t *error);
    
    static void listenCallback( void *args, globus_ftp_control_server_t *handle,
            globus_object_t *error );

    static void authCallback( void *args, globus_ftp_control_handle_t *handle,
            globus_object_t *error, globus_ftp_control_auth_info_t * result );
    
    static void acceptCallback( void *args, globus_ftp_control_handle_t *handle,
        globus_object_t *error );

    static void readCommands( void *args, globus_ftp_control_handle_t *handle,
        globus_object_t *error, 
        globus_ftp_control_command_t *command ); 

    static int getAuthRequirements( );
    
    void assignRequest( nestRequest *request );

    nestRequest * getRequest( );
    
    static void lockFifo();

    static void unlockFifo();

    static int enQueueConnection( nestProtocol *connection );

    static nestProtocol * deQueueConnection( );
    
    virtual bool sendAck(  );

    virtual bool sendMessage(  const void *vptr );
    globus_result_t closeDataChannel( );

};

#endif
