/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
  *
  * Condor Software Copyright Notice
  * Copyright (C) 1990-2005, Condor Team, Computer Sciences Department,
  * University of Wisconsin-Madison, WI.
  *
  * This source code is covered by the Condor Public License, which can
  * be found in the accompanying LICENSE.TXT file, or online at
  * www.condorproject.org.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  * AND THE UNIVERSITY OF WISCONSIN-MADISON "AS IS" AND ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
  * WARRANTIES OF MERCHANTABILITY, OF SATISFACTORY QUALITY, AND FITNESS
  * FOR A PARTICULAR PURPOSE OR USE ARE DISCLAIMED. THE COPYRIGHT
  * HOLDERS AND CONTRIBUTORS AND THE UNIVERSITY OF WISCONSIN-MADISON
  * MAKE NO MAKE NO REPRESENTATION THAT THE SOFTWARE, MODIFICATIONS,
  * ENHANCEMENTS OR DERIVATIVE WORKS THEREOF, WILL NOT INFRINGE ANY
  * PATENT, COPYRIGHT, TRADEMARK, TRADE SECRET OR OTHER PROPRIETARY
  * RIGHT.
  *
  ****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/

#ifndef _NEST_PROTOCOL_NFS_H
#define _NEST_PROTOCOL_NFS_H

#include <queue>

#include "nest_common.h"
#include "nest_protocol.h"

#include "nfs_prot.h"

class nestProtocolNFS : public nestProtocol {
public:
    static int listen( int port );

    static nestClient* accept( int socket, 
                                fd_set *readies, 
                                int accepts );
    
    nestProtocolNFS(   SVCXPRT    *transp, 
                        xdrproc_t xdr_result,
                        xdrproc_t xdr_argument,
                        caddr_t   argument,
                        int       general_type,
                        int       request_type );

    virtual ~nestProtocolNFS();

    virtual bool persistent();
    
    virtual bool sendFinalAck( int transferType );
    
    virtual bool sendAck(  );

    virtual bool sendTransferInfo( nestRequest* request );
    
    virtual bool sendErrorMessage(  const char* filename,
									const char* func_name,
									int error );
    
    virtual NestReplyStatus sendFileinfo( const NestFileInfo *info );

    virtual NestReplyStatus setFileattr( NestFileInfo *info,
										 const char *fname,
										 const void *fattr );

    virtual bool sendDiskspace( nestLotSize avail, nestLotSize total );
    
    virtual bool sendDirectoryList( NestFileInfo *list );
    
    virtual bool sendFilesize( nestFileSize size);
    
    virtual nestRequest* receiveRequest(  fd_set *ready );

    virtual ssize_t Receive( char *buffer, ssize_t n );

    virtual ssize_t Send( const char *buffer, ssize_t n );
    
    virtual int getType( ) { return NFS_PROTOCOL; }

    virtual bool hasSocket();

    NestReplyStatus sendFilehandle( NestFileInfo *handle );

    int getSocket();

	const char *getName( void ) { return "NFS"; };
    
private:

    static void newconnection(  nestRequest *,
                                SVCXPRT *, 
                                xdrproc_t, 
                                xdrproc_t, 
                                caddr_t,
                                svc_req *, 
                                int );
    
    static void request_handler( svc_req *, SVCXPRT * );
    
    static void nfs_request_handler( svc_req *, SVCXPRT * );

    static void mount_request_handler( svc_req *, SVCXPRT * );
    
    static char *reverseLookup( nfs_fh *fh );
    
    static int pipes[2];

    static queue< nestClient *>clients; 

    NestReplyStatus sendReply( char *result );
    
    NestReplyStatus setFileAttributes( fattr *attributes, NestFileInfo *info );

    void lockMutex();
    void unlockMutex();
    
    SVCXPRT     *transp;
    xdrproc_t   xdr_result;
    xdrproc_t   xdr_argument;
    caddr_t     argument; 
    int         gen_type;
    int         req_type;
};

#endif
