/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
  *
  * Condor Software Copyright Notice
  * Copyright (C) 1990-2005, Condor Team, Computer Sciences Department,
  * University of Wisconsin-Madison, WI.
  *
  * This source code is covered by the Condor Public License, which can
  * be found in the accompanying LICENSE.TXT file, or online at
  * www.condorproject.org.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  * AND THE UNIVERSITY OF WISCONSIN-MADISON "AS IS" AND ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
  * WARRANTIES OF MERCHANTABILITY, OF SATISFACTORY QUALITY, AND FITNESS
  * FOR A PARTICULAR PURPOSE OR USE ARE DISCLAIMED. THE COPYRIGHT
  * HOLDERS AND CONTRIBUTORS AND THE UNIVERSITY OF WISCONSIN-MADISON
  * MAKE NO MAKE NO REPRESENTATION THAT THE SOFTWARE, MODIFICATIONS,
  * ENHANCEMENTS OR DERIVATIVE WORKS THEREOF, WILL NOT INFRINGE ANY
  * PATENT, COPYRIGHT, TRADEMARK, TRADE SECRET OR OTHER PROPRIETARY
  * RIGHT.
  *
  ****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/

#include "nest_common.h"
#include "nest_pthread_util.h"
#include "network_limit.h"

#include <stdio.h>
#include <stdlib.h>
#include <pthread.h>

static void * thread_start( void *arg )
{
    NetworkLimit *n = (NetworkLimit*)arg;
    n->run();
    return 0;
}

NetworkLimit::NetworkLimit( )
{
    pthread_t tid;

    pthread_mutex_init(&mutex,0);
    pthread_cond_init(&cond,0);

    bandwidth = 0;
    interval = 50000;

    nest_pthread_create(&tid,0,thread_start,this);
    pthread_detach(tid);
}

void NetworkLimit::run()
{
    while(1) {
        pthread_mutex_lock(&mutex);
        available = (unsigned long)(((float)bandwidth)*interval/MILLION);
        pthread_cond_broadcast(&cond);
        pthread_mutex_unlock(&mutex);
        NestNanosleep(interval*1000);
    }
}

void NetworkLimit::setBandwidth( unsigned long b )
{
        bandwidth = b;
        nest_printf( "Bandwidth limited to %ld bytes/sec\n", bandwidth );
}

void NetworkLimit::setInterval( unsigned long i )
{
    interval = i;
}

void NetworkLimit::consumeBandwidth( unsigned long bytes )
{
    if ( bandwidth != 0 ) {
        pthread_mutex_lock(&mutex);
        while(bytes>0) {
            if( available>=bytes ) {
                available -= bytes;
                bytes = 0;
                break;
            } else {
                bytes -= available;
                available = 0;
            }
            pthread_cond_wait(&cond,&mutex);
        }
        pthread_mutex_unlock(&mutex);
    }
}
